import os

from django.http import Http404
from django.utils.translation import to_locale, get_language
from django.contrib.gis.geoip2 import GeoIP2
from django.conf import settings
from .models import Copy

#BASE_PATH = os.path.dirname(os.path.abspath(__file__))
#GEOIP_PATH = os.path.join(BASE_PATH,'geoip2')
GEOIP_PATH = settings.GEOIP_PATH


def get_ip_address(request):
    "Map request to external IP address resolving internal address if necessary"

    forwarded = request.META.get('HTTP_X_FORWARDED_FOR')

    if forwarded:
        ip = forwarded.split(',')[0]
    else:
        ip = request.META.get('REMOTE_ADDR')

    return ip



def ip_to_country_code(addr, default_code='GB'):
    "Map request to client's country based on IP address"

    if addr == '127.0.0.1' or addr == 'localhost':
        return default_code

    g = GeoIP2(path=GEOIP_PATH)
    return g.country(addr)['country_code']



def get_client_country_code(request):
    "Shorthand to request country code directly from request"
    return ip_to_country_code(get_ip_address(request))



def LocalisationMiddleware(get_response):

    def middleware(request):
        url = request.path
        locale = to_locale(get_language())
        geo = get_client_country_code(request)
        draft = request.user.is_authenticated and 'draft' in request.GET

        request.copy = Copy.get_for_url(url, locale, geo, draft)

        return get_response(request)

    return middleware



# Generated by Django 2.0.10 on 2019-08-10 14:24

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangocopy', '0005_auto_20190807_2333'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image',
            name='image',
            field=models.ImageField(upload_to='djangocopy/images/'),
        ),
        migrations.AlterField(
            model_name='template',
            name='template',
            field=models.FileField(upload_to='djangocopy/templates/'),
        ),
    ]



# Generated by Django 2.0.10 on 2019-08-07 22:33

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangocopy', '0004_auto_20190807_2329'),
    ]

    operations = [
        migrations.AlterField(
            model_name='image',
            name='label',
            field=models.CharField(blank=True, max_length=255),
        ),
        migrations.AlterField(
            model_name='template',
            name='label',
            field=models.CharField(blank=True, max_length=255),
        ),
    ]



# Generated by Django 2.0.10 on 2019-08-06 12:08

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('djangocopy', '0002_historicalcopy_historicalnavbar_historicalpage'),
    ]

    operations = [
        migrations.AddField(
            model_name='historicalnavbar',
            name='label',
            field=models.CharField(default='Blank', max_length=255),
            preserve_default=False,
        ),
        migrations.AddField(
            model_name='navbar',
            name='label',
            field=models.CharField(default='Blank', max_length=255),
            preserve_default=False,
        ),
    ]



from django.http import Http404
from django.shortcuts import render, redirect
from django.utils.decorators import decorator_from_middleware

from .middleware import LocalisationMiddleware
from .models import Page


@decorator_from_middleware(LocalisationMiddleware)
def static_page(request,slug):
    try:
        page = Page.objects.get(slug=slug)
        if page.authenticated and not request.user.is_authenticated: raise Http404
        
        template = page.template.template.name
        metadata = {
            'title':        page.title,
            'description':  page.description,
            'keywords':     page.keywords,
        }

    except:
        raise Http404

    return render(request, "djangocopy/wrapper.html", context={'template': template, 'metadata': metadata})



def index(request):
    u = request.user
    if u.is_superuser or u.is_staff:
        return redirect('admin:index')
    else:
        return render(request, "djangocopy/default.html")



class BasicView():
    FORM_CLASS = None

    def get(self, request, *args, **kwargs):
        form = self.FORM_CLASS(request.GET)
        if form.is_valid():
            return self.form_is_valid(form)
        else:
            return self.form_is_not_valid(form)

    def post(self, request, *args, **kwargs):
        form = self.FORM_CLASS(request.POST)
        if form.is_valid():
            return self.form_is_valid(form)
        else:
            return self.form_is_not_valid(form)

    def unbound(self, request, *args, **kwargs):
        pass

    def form_is_valid(self, form):
        pass

    def form_is_not_valid(self, form):
        pass

    def __call__(self, request, *args, **kwargs):
        if request.GET:
            return self.get(request, *args, **kwargs)
        
        elif request.POST:
            return self.post(request, *args, **kwargs)

        else:
            return self.unbound(request, *args, **kwargs)


import os
from django.urls import path, re_path
from django.conf import settings
from django.views.static import serve

from .views import static_page, index


def unprotected_serve(request, path):
    "Serve django-copy templates and images without user restrictions that may be imposed on MEDIA_URL"
    return serve(request, path, settings.MEDIA_ROOT, True)



urlpatterns = [
    path('<str:slug>/',static_page,name='static'),
    path('', index),
]

# If DEBUG is False, the MEDIA_URL/djangocopy/* urls are not served by Django
#if settings.DEBUG:
#    urlpatterns += [
#        #re_path(r'^%sdjangocopy/(?P<path>.*)$' % settings.MEDIA_URL[1:], unprotected_serve, {'document_root': settings.MEDIA_ROOT}),
#        re_path(r'^{}/(?P<path>.*)$'.format(settings.DJANGOCOPY_IMAGE_ROOT[1:]), unprotected_serve),
#    ]


# Generated by Django 2.0.10 on 2019-08-07 22:29

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('djangocopy', '0003_auto_20190806_1308'),
    ]

    operations = [
        migrations.CreateModel(
            name='Image',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=255)),
                ('image', models.ImageField(upload_to='djangocopy/images')),
            ],
        ),
        migrations.AlterField(
            model_name='historicalnavbar',
            name='logo',
            field=models.ForeignKey(blank=True, db_constraint=False, help_text='A picture to use as a logo', null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='+', to='djangocopy.Image'),
        ),
        migrations.AlterField(
            model_name='navbar',
            name='logo',
            field=models.ForeignKey(blank=True, help_text='A picture to use as a logo', null=True, on_delete=django.db.models.deletion.SET_NULL, to='djangocopy.Image'),
        ),
        migrations.AlterField(
            model_name='template',
            name='template',
            field=models.FileField(upload_to='djangocopy/templates'),
        ),
    ]



# Generated by Django 2.1.1 on 2018-10-12 06:56

from django.db import migrations, models
import django.db.models.deletion
import jsonfield.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='Copy',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('url', models.CharField(blank=True, help_text='URL name (leave empty to load for all templates)', max_length=255)),
                ('fieldid', models.SlugField(help_text='The field identifier that will be used in templates', max_length=100)),
                ('locale', models.CharField(blank=True, help_text="Browser settings (e.g. 'en_GB')", max_length=5)),
                ('geo', models.CharField(blank=True, help_text="Country code derived from the IP (e.g. 'GB')", max_length=2)),
                ('text', models.TextField(max_length=10000)),
                ('format', models.CharField(choices=[('p', 'Plain text'), ('m', 'Markdown'), ('j', 'JSON'), ('h', 'HTML'), ('s', 'Special HTML')], default='p', max_length=1)),
                ('status', models.CharField(choices=[('d', 'Draft'), ('p', 'Published')], default='d', max_length=1)),
            ],
            options={
                'verbose_name_plural': 'copy',
            },
        ),
        migrations.CreateModel(
            name='Navbar',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('logo', models.ImageField(blank=True, help_text='A picture to use as a logo', null=True, upload_to='djangocopy/')),
                ('elements', jsonfield.fields.JSONField()),
                ('z_index', models.IntegerField(default=0, help_text='The z-index determines the order of navbar items. A higher value appears first.')),
                ('groups', models.ManyToManyField(blank=True, help_text='Associate navbar with a particular user group.', to='auth.Group')),
            ],
        ),
        migrations.CreateModel(
            name='Page',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('slug', models.SlugField(max_length=255, unique=True)),
                ('authenticated', models.BooleanField(default=False, help_text='If True, visitor must login to access this page')),
                ('title', models.CharField(blank=True, default='', max_length=255)),
                ('description', models.CharField(blank=True, default='', max_length=255)),
                ('keywords', models.CharField(blank=True, default='', max_length=255)),
            ],
        ),
        migrations.CreateModel(
            name='Template',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('label', models.CharField(max_length=255)),
                ('template', models.FileField(upload_to='templates')),
            ],
        ),
        migrations.AddField(
            model_name='page',
            name='template',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, to='djangocopy.Template'),
        ),
        migrations.AlterUniqueTogether(
            name='copy',
            unique_together={('fieldid', 'url', 'locale', 'geo', 'status')},
        ),
    ]



from markdown import markdown
from json import loads
from jsonfield import JSONField

from django.db import models
from django.db.models import Q
from django.urls import resolve
from django.conf import settings
from django.utils.safestring import mark_safe
from django.contrib.auth.models import Group

from simple_history.models import HistoricalRecords



class Template(models.Model):
    "HTML Templates"

    template = models.FileField(upload_to=settings.DJANGOCOPY_TEMPLATES)
    label = models.CharField(max_length=255, blank=True)

    def __str__(self):
        return self.label if self.label else self.template.name


class Image(models.Model):
    "Image Files"

    image = models.ImageField(upload_to=settings.DJANGOCOPY_IMAGES)
    label = models.CharField(max_length=255, blank=True)

    def __str__(self):
        return self.label if self.label else self.image.name


class Page(models.Model):
    "Definition for pages managed by the djangocopy"

    slug = models.SlugField(max_length=255, unique=True)
    template = models.ForeignKey(Template, on_delete=models.PROTECT)
    authenticated = models.BooleanField(default=False, help_text="If True, visitor must login to access this page")

    title = models.CharField(max_length=255,default='', blank=True)
    description = models.CharField(max_length=255,default='', blank=True)
    keywords = models.CharField(max_length=255,default='', blank=True)

    history = HistoricalRecords()

    @property
    def get_title(self):
        return self.title if self.title else self.slug


class Navbar(models.Model):
    "Navbar links"

    label = models.CharField(max_length=255)
    logo = models.ForeignKey(Image, on_delete=models.SET_NULL, null=True, blank=True, help_text="A picture to use as a logo")
    groups = models.ManyToManyField(Group, blank=True, help_text="Associate navbar with a particular user group.")
    elements = JSONField()
    z_index = models.IntegerField(default=0, help_text="The z-index determines the order of navbar items. A higher value appears first.")

    history = HistoricalRecords()


class Copy(models.Model):
    "djangocopy content"

    FORMAT_PLAIN = 'p'
    FORMAT_MARKDOWN = 'm'
    FORMAT_JSON = 'j'
    FORMAT_SAFE_HTML = 'h'
    FORMAT_SPECIAL_HTML = 's'

    FORMAT_CHOICES = (
        (FORMAT_PLAIN,'Plain text'),
        (FORMAT_MARKDOWN,'Markdown'),
        (FORMAT_JSON,'JSON'),
        (FORMAT_SAFE_HTML,'HTML'),
        (FORMAT_SPECIAL_HTML,'Special HTML'),
    )

    STATUS_DRAFT = 'd'
    STATUS_PUBLISHED = 'p'

    STATUS_CHOICES = (
        (STATUS_DRAFT,'Draft'),
        (STATUS_PUBLISHED,'Published'),
    )

    url = models.CharField(max_length=255,blank=True,help_text="URL name (leave empty to load for all templates)")
    fieldid = models.SlugField(max_length=100,help_text="The field identifier that will be used in templates")

    locale = models.CharField(max_length=5,blank=True,help_text="Browser settings (e.g. 'en_GB')")
    geo = models.CharField(max_length=2,blank=True,help_text="Country code derived from the IP (e.g. 'GB')")

    text = models.TextField(max_length=10000)
    format = models.CharField(max_length=1,choices=FORMAT_CHOICES,default=FORMAT_PLAIN)
    status = models.CharField(max_length=1,choices=STATUS_CHOICES,default=STATUS_DRAFT)

    history = HistoricalRecords()

    class Meta:
        verbose_name_plural = 'copy'
        unique_together = (("fieldid","url","locale","geo","status"),)

    @property
    def status_as_string(self):
        return dict(self.STATUS_CHOICES).get(self.status, '--')

    def render(self):
        return __MAPPING__[self.format](self.text)

    def short_text(self,truncate=80):
        return self.text[:truncate] + '...' if len(self.text) > truncate else self.text

    def __str__(self):
        str = "{}/{} - {}".format(self.url or '*',self.fieldid,self.text[:40])
        if self.locale or self.geo: str += " ({},{})".format(self.locale or '*',self.geo or '*')
        str += ' ' + self.status_as_string.upper()
        return str

    @staticmethod
    def get_for_url(url, locale, geo, draft=False):
        """Retrieve set of copies for a given URL (or URL name), locale and geographical location"""
        if draft:
            qs = Copy.objects.filter(
                Q(url=url) | Q(url=resolve(url).url_name) | Q(url=''),
                Q(locale=locale) | Q(locale=locale[:2]) | Q(locale=''),
                Q(geo=geo) | Q(geo=''),
            ).order_by('locale', 'geo', '-status') # Ordering puts defaults first so they get overwritten when converting to dict

        else:
            qs = Copy.objects.filter(
                Q(url=url) | Q(url=resolve(url).url_name) | Q(url=''),
                Q(locale=locale) | Q(locale=locale[:2]) | Q(locale=''),
                Q(geo='') | Q(geo=geo),
                status = Copy.STATUS_PUBLISHED,
            ).order_by('locale', 'geo') # Ordering puts defaults first so they get overwritten when converting to dict

        values = qs.values_list('fieldid', 'text', 'format')
        out = [(fld, __MAPPING__[fmt](txt)) for fld, txt, fmt in values]
        return dict(out)



def __map_json__(txt):
    try:
        return loads(txt)
    except Exception as e:
        logging.error('Error decoding JSON copy: {} ({})'.format(txt,e))
        pass # log error and fail silently


__MAPPING__ = {
    Copy.FORMAT_PLAIN: lambda txt: txt,
    Copy.FORMAT_MARKDOWN: lambda txt: mark_safe(markdown(txt)),
    Copy.FORMAT_JSON: lambda txt: __map_json__(txt),
    Copy.FORMAT_SAFE_HTML: lambda txt: mark_safe(txt),
    Copy.FORMAT_SPECIAL_HTML: lambda txt: mark_safe(txt),
}




#!/usr/bin/env python
import os
from setuptools import setup,find_packages

PATH = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(PATH, 'README.md'), encoding='utf-8') as fp:
    DESC = fp.read()


setup(
    name='django-copy',
    version='0.1.0',
    description='Probably the smallest and simplest CMS for the Django framework',
    long_description=DESC,
    long_description_content_type = 'text/markdown',
    keywords='Django, CMS',
    author='Pedro Tavares',
    author_email='web@ptavares.com',
    url='https://github.com/ptav/django-copy',
    license='LICENSE',

    packages=find_packages(),
    include_package_data=True,
    
    classifiers=[
        "Programming Language :: Python",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Framework :: Django",
        "Environment :: Web Environment",
        "Development Status :: 4 - Beta",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.6",
    ],
)



from django import forms
from django.shortcuts import redirect
from django.contrib import admin

from ckeditor_uploader.widgets import CKEditorUploadingWidget

from .models import Template, Image, Page, Copy, Navbar



ELEMENTS_HELPTEXT = \
"""
Simple entry: { 'label':"Home", url:"/" }
Add image: { 'label':"Logout", url:"/accounts/logout", img:"/avatar.jpg" }
Add FA icon: { 'label':"Login", url:"/accounts/login", faicon:"fa-signin" }
Dropdown (and example of divider):  
    {
        "label":" ",
        "img":"/game/useravatar",
        "dropdown":[
            { "label":"Profile", "url":"/game/profile", "divider":1 },
            { "label":"Sign-out", "url":"/accounts/logout" }
        ]
    }

"""

@admin.register(Navbar)
class NavbarAdmin(admin.ModelAdmin):
    model = Navbar

    list_display = ('label', 'get_groups', )
    autocomplete_fields = ('groups', )

    help_texts = {
        'elements': ELEMENTS_HELPTEXT,
    }

    def get_groups(self, obj):
        txt = ""
        for grp in obj.groups.all():
            txt += ', ' + grp.name
        return txt[2:] if len(txt) > 2 else ""
    get_groups.short_description = 'Groups'


@admin.register(Page)
class PageAdmin(admin.ModelAdmin):
    model = Page

    ordering= ('slug', )
    list_display = ('id', 'slug', 'template', )
    search_fields = ('slug', )

    fieldsets = (
        (None, {
            'fields': ('slug', 'template', )
        }),
        ('SEO', {
            #'classes': ('collapse', ),
            'fields': ('title', 'description', 'keywords', ),
        }),
    )



def publish_drafts(modeladmin, request, queryset):
    for obj in queryset:
        pub, created = Copy.objects.get_or_create(
            url=obj.url,
            fieldid=obj.fieldid,
            locale=obj.locale,
            geo=obj.geo,
            status=Copy.STATUS_PUBLISHED,
        )

        pub.text = obj.text
        pub.format = obj.format
        pub.save()

        obj.delete()

    return redirect(request.path)



class CopyAdminForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super(CopyAdminForm, self).__init__(*args, **kwargs)

        if self.instance is None or self.instance.format == Copy.FORMAT_SAFE_HTML:
            self.fields['text'].widget = CKEditorUploadingWidget()

    class Meta:
        model = Copy
        fields = '__all__'



@admin.register(Copy)
class CopyAdmin(admin.ModelAdmin):
    model = Copy
    form = CopyAdminForm
    actions = (publish_drafts, )

    ordering= ('url', 'fieldid', 'locale', 'geo', '-status')
    list_display = ('url', 'fieldid', 'locale', 'geo', 'short_text', 'format', 'status')
    list_filter = ('status', 'locale', 'geo', 'url', 'fieldid')
    search_fields = ('text', 'locale', 'geo', 'fieldid', 'url')


admin.site.register(Template)
admin.site.register(Image)